#!/usr/bin/env python

import argparse
import logging
import os
import sys

from distutils.spawn import find_executable
from subprocess import call


def parse_args():
    arg_parser = argparse.ArgumentParser()

    arg_parser.add_argument(
        '-t', '--target',
        help='Target to test')

    arg_parser.add_argument(
        '--valgrind', default=False,
        help='Run with Valgrind',
        action='store_true')

    return arg_parser.parse_args(sys.argv[1:])


def get_system_exe_extension():
    # Should return 'win32' even on 64-bit Windows
    if sys.platform == 'win32':
        return '.exe'
    else:
        return ''


def find_exe(name, path):
    for root, dirs, files in os.walk(path):
        if name in files:
            return os.path.join(root, name)


def build_test_exe_args(args, build_dir):
    if args.target is None:
        logging.error('Target not specified, please use the --target option')
        return None

    test_exe = find_exe(args.target + get_system_exe_extension(), build_dir)
    if not os.path.exists(test_exe):
        logging.error('Could not find test executable for target {}, '
                      'did you forget to build?'.format(args.target))
    else:
        logging.debug('Test executable is: {}'.format(test_exe))

    test_exe_args = [test_exe]

    if args.valgrind is not False:
        valgrind_exe = find_executable('valgrind')
        if valgrind_exe is None:
            logging.error('Valgrind not found, cannot continue')
            return None

        test_exe_args = [
            valgrind_exe, '--leak-check=full', '--show-reachable=yes',
            '--gen-suppressions=all', '--error-exitcode=1', '--track-origins=yes',
            '--suppressions=../ci/memcheck.supp', test_exe]

    return test_exe_args


def run_tests(args):
    scripts_dir = os.path.dirname(os.path.realpath(__file__))
    root_dir = os.path.join(scripts_dir, os.pardir)
    build_dir = os.path.join(root_dir, 'build')
    if not os.path.exists(build_dir):
        logging.error(
            'Build directory not found, did you forget to run the configure.py script?')
        return 2

    os.chdir(build_dir)
    env = os.environ.copy()
    env['GLIBCXX_FORCE_NEW'] = '1'

    test_exe_args = build_test_exe_args(args, build_dir)
    if test_exe_args is None:
        return 1

    logging.info(test_exe_args)
    logging.info('Running Tests for {}'.format(args.target))
    return call(test_exe_args, env=env)


if __name__ == '__main__':
    logging.basicConfig(format='%(message)s', level=logging.INFO, stream=sys.stdout)
    sys.exit(run_tests(parse_args()))



#!/usr/bin/env python

import argparse
import logging
import os
import sys

from distutils.spawn import find_executable
from subprocess import call


def parse_args():
    arg_parser = argparse.ArgumentParser()

    arg_parser.add_argument(
        '--cmake',
        default=find_executable("cmake"),
        help='Path to CMake executable (default: %(default)s)')

    arg_parser.add_argument(
        '-c', '--configuration',
        help='Build configuration to use (not supported by IDE generators)')

    arg_parser.add_argument(
        '-a', '--arguments',
        help='Arguments to pass to builder')

    return arg_parser.parse_args(sys.argv[1:])


def build_cmake_args(args, build_dir):
    if args.cmake is None:
        logging.error('CMake not found, please use the --cmake option')
        return None

    cmake_args = []
    cmake_args.append(args.cmake)
    cmake_args.append('--build')
    cmake_args.append(build_dir)

    if args.configuration is not None:
        cmake_args.append('--config')
        cmake_args.append(args.configuration)

    if args.arguments is not None:
        cmake_args.append('--')
        for arg in args.arguments.split():
            cmake_args.append(arg)

    return cmake_args


def build(args):
    scripts_dir = os.path.dirname(os.path.realpath(__file__))
    root_dir = os.path.join(scripts_dir, os.pardir)
    build_dir = os.path.join(root_dir, 'build')
    if not os.path.exists(build_dir):
        logging.error(
            'Build directory not found, did you forget to run the configure.py script?')
        return 2

    cmake_args = build_cmake_args(args, build_dir)
    if cmake_args is None:
        return 1

    logging.info('Running CMake')
    return call(cmake_args)


if __name__ == '__main__':
    logging.basicConfig(format='%(message)s', level=logging.INFO, stream=sys.stdout)
    sys.exit(build(parse_args()))



#!/usr/bin/env python

import argparse
import logging
import os
import shutil
import sys

from distutils.spawn import find_executable
from subprocess import call


def parse_args():
    arg_parser = argparse.ArgumentParser()

    arg_parser.add_argument(
        '-a', '--audio-driver',
        help='Audio driver to build (Windows only, default: %(default)s)')

    arg_parser.add_argument(
        '--cmake',
        default=find_executable("cmake"),
        help='Path to CMake executable (default: %(default)s)')

    arg_parser.add_argument(
        '-c', '--configuration',
        help='Build configuration to use (not supported by IDE generators)')

    arg_parser.add_argument(
        '-g', '--generator',
        help='CMake generator to use (default: Determined by CMake)')

    arg_parser.add_argument(
        '-f', '--flags',
        help='Additional CMake flags')

    arg_parser.add_argument(
        '--thread-description',
        help='Set thread description. (Windows only)')

    return arg_parser.parse_args(sys.argv[1:])


def build_cmake_args(args):
    if args.cmake is None:
        logging.error('CMake not found, please use the --cmake option')
        return None

    cmake_args = []
    cmake_args.append(args.cmake)

    if args.generator is not None:
        cmake_args.append('-G')
        cmake_args.append(args.generator)

    if args.configuration is not None:
        cmake_args.append('-DCMAKE_BUILD_TYPE=' + args.configuration)

    if args.flags is not None:
        cmake_args.append(args.flags)

    if sys.platform == 'win32':
        if args.audio_driver is None or args.audio_driver == 'Asio':
            cmake_args.append('-DLINK_BUILD_ASIO=ON')
        else:
            cmake_args.append('-DLINK_BUILD_ASIO=OFF')
        if args.thread_description == 'ON':
            cmake_args.append('-DLINK_WINDOWS_SETTHREADDESCRIPTION=ON')
    elif 'linux' in sys.platform:
       if args.audio_driver == 'Jack':
           cmake_args.append('-DLINK_BUILD_JACK=ON')

    # This must always be last
    cmake_args.append('..')
    return cmake_args


def configure(args):
    scripts_dir = os.path.dirname(os.path.realpath(__file__))
    root_dir = os.path.join(scripts_dir, os.pardir)
    build_dir = os.path.join(root_dir, 'build')
    if os.path.exists(build_dir):
        logging.info('Removing existing build directory')
        shutil.rmtree(build_dir)

    logging.debug('Creating build directory')
    os.mkdir(build_dir)
    os.chdir(build_dir)

    cmake_args = build_cmake_args(args)
    if cmake_args is None:
        return 1

    logging.info('Running CMake')
    return call(cmake_args)


if __name__ == '__main__':
    logging.basicConfig(format='%(message)s', level=logging.INFO, stream=sys.stdout)
    sys.exit(configure(parse_args()))



#!/usr/bin/env python

import argparse
import logging
import os
import subprocess
import sys


def parse_args():
    arg_parser = argparse.ArgumentParser()

    arg_parser.add_argument(
        '-c', '--clang-format',
        default='clang-format-6.0',
        help='Path to clang-format executable')

    arg_parser.add_argument(
        '-f', '--fix', action='store_true',
        help='Automatically fix all files with formatting errors')

    return arg_parser.parse_args(sys.argv[1:])


def parse_clang_xml(xml):
    for line in xml.splitlines():
        if line.startswith(b'<replacement '):
            return False
    return True


def fix_file(args, file_absolute_path):
    logging.info('Fixing formatting errors in file: {}'.format(file_absolute_path))
    clang_format_args = [args.clang_format, '-style=file', '-i', file_absolute_path]
    try:
        subprocess.check_call(clang_format_args)
    except subprocess.CalledProcessError:
        logging.error('Error running clang-format on {},'
                      ' please run clang-format -i by hand'.format(file_absolute_path))


def check_files_in_path(args, path):
    logging.info('Checking files in {}'.format(path))
    errors_found = False

    for (path, dirs, files) in os.walk(path):
        for file in files:
            if file.endswith(('.c', '.cpp', '.h', '.hpp', '.ipp')):
                file_absolute_path = path + os.path.sep + file
                clang_format_args = [
                    args.clang_format, '-style=file',
                    '-output-replacements-xml', file_absolute_path]

                try:
                    clang_format_output = subprocess.check_output(clang_format_args)
                except subprocess.CalledProcessError:
                    logging.error(
                        'Could not execute {}, try running this script with the'
                        '--clang-format option'.format(args.clang_format))
                    sys.exit(2)

                if not parse_clang_xml(clang_format_output):
                    if args.fix:
                        fix_file(args, file_absolute_path)
                    else:
                        logging.warning(
                            '{} has formatting errors'.format(file_absolute_path))
                        errors_found = True

    return errors_found


def check_formatting(args):
    errors_found = False
    script_dir = os.path.dirname(os.path.realpath(__file__))
    for path in ['../examples', '../include', '../src', '../extensions/abl_link']:
        subdir_abs_path = os.path.abspath(os.path.join(script_dir, path))
        if check_files_in_path(args, subdir_abs_path):
            errors_found = True

    if errors_found:
        logging.warning(
            'Formatting errors found, please fix with clang-format -style=file -i')
    else:
        logging.debug('No formatting errors found!')

    return errors_found


if __name__ == '__main__':
    logging.basicConfig(format='%(message)s', level=logging.INFO, stream=sys.stdout)
    sys.exit(check_formatting(parse_args()))
